// Generated by CoffeeScript 1.6.3
/*

  (S)ync - simply serialize, defer, and join async calls
  
  c) 2013 Sebastian Glaser <anx@ulzq.de>

  This file is part of the ync project.

  ync is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2, or (at your option)
  any later version.

  ync is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this software; see the file COPYING.  If not, write to
  the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  Boston, MA 02111-1307 USA

  http://www.gnu.org/licenses/gpl.html
*/

var EventEmitter, Join, Sync,
  __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
  __hasProp = {}.hasOwnProperty,
  __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

if ((typeof module !== "undefined" && module !== null) && (module.exports != null)) {
  EventEmitter = require('events').EventEmitter;
} else {
  EventEmitter = (function() {
    function _Class() {}

    return _Class;

  })();
}

Object.merge = function(t, d) {
  var k, v;
  for (k in d) {
    v = d[k];
    t[k] = d[k];
  }
  return t;
};

Object.defaults = function(t, d) {
  var k, v;
  for (k in d) {
    v = d[k];
    if (t[k] == null) {
      t[k] = d[k];
    }
  }
  return t;
};

Object.snatch = function(o, k) {
  var f, t, _i, _len, _results;
  if (Array.isArray(k)) {
    _results = [];
    for (_i = 0, _len = k.length; _i < _len; _i++) {
      f = k[_i];
      t = o[f];
      delete o[f];
      _results.push(t);
    }
    return _results;
  } else {
    t = o[k];
    delete o[k];
    return t;
  }
};

Sync = (function(_super) {
  __extends(Sync, _super);

  Sync.count = 0;

  function Sync(chain) {
    var run, _ref;
    this.chain = chain;
    this.count = __bind(this.count, this);
    this.last = __bind(this.last, this);
    this.first = __bind(this.first, this);
    this.next = __bind(this.next, this);
    this.insertAfter = __bind(this.insertAfter, this);
    this.insertBefore = __bind(this.insertBefore, this);
    this.proceed = __bind(this.proceed, this);
    this.exec = __bind(this.exec, this);
    this.run = __bind(this.run, this);
    this.id = Sync.count++;
    this.chain = Object.defaults(this.chain, {
      debug: false,
      fork: false,
      run: true,
      title: "Sync-" + this.id
    });
    _ref = Object.snatch(this.chain, ['debug', 'fork', 'run', 'title', 'onexec', 'end']), this.debug = _ref[0], this.fork = _ref[1], run = _ref[2], this.title = _ref[3], this.onexec = _ref[4], this.end = _ref[5];
    if (this.debug) {
      require('colors');
      console.log("new".yellow, "Sync".blue, this.title.red, Object.keys(this.chain));
    }
    this.current = Object.keys(this.chain).shift();
    if (run) {
      this.exec(this.current, []);
    }
  }

  Sync.prototype.run = function(rule) {
    if (rule != null) {
      this.current = rule;
    }
    return this.exec(this.current, []);
  };

  Sync.prototype.exec = function(rule, args) {
    var _this = this;
    if (this.debug) {
      console.log("Executing".yellow, rule, args);
    }
    if (this.onexec) {
      this.onexec(rule, this.chain[rule]);
    }
    if (this.fork) {
      setTimeout((function() {
        return _this.chain[rule].apply(_this, args);
      }), 0);
    } else {
      this.chain[rule].apply(this, args);
    }
    if ((this.end != null) && this.current === Object.keys(this.chain).pop()) {
      return this.end();
    }
  };

  Sync.prototype.proceed = function() {
    var args, k, v;
    if (this.current === this.last()) {
      for (k in arguments) {
        v = arguments[k];
        args = v;
      }
      return this.emit.apply(this, ['done'].concat(args));
    }
    this.current = this.next();
    return this.exec(this.current, arguments);
  };

  Sync.prototype.insertBefore = function(what, name, func) {
    var k, keys, l, s, _i, _len;
    s = this.chain;
    l = {};
    keys = Object.keys(s);
    keys = keys.concat(name, keys.splice(keys.indexOf(what)));
    s[name] = func;
    for (_i = 0, _len = keys.length; _i < _len; _i++) {
      k = keys[_i];
      l[k] = s[k];
    }
    return this.chain = l;
  };

  Sync.prototype.insertAfter = function(what, name, func) {
    var k, keys, l, s, _i, _len;
    s = this.chain;
    l = {};
    keys = Object.keys(s);
    keys = keys.concat(name, keys.splice(keys.indexOf(what) + 1));
    s[name] = func;
    for (_i = 0, _len = keys.length; _i < _len; _i++) {
      k = keys[_i];
      l[k] = s[k];
    }
    return this.chain = l;
  };

  Sync.prototype.next = function() {
    var i, k;
    k = Object.keys(this.chain);
    i = k.indexOf(this.current);
    if (i + 1 < k.length) {
      return k[i + 1];
    }
  };

  Sync.prototype.first = function() {
    return Object.keys(this.chain).shift();
  };

  Sync.prototype.last = function() {
    return Object.keys(this.chain).pop();
  };

  Sync.prototype.count = function() {
    return Object.keys(this.chain).length;
  };

  return Sync;

})(EventEmitter);

Join = (function() {
  Join.ids = 0;

  function Join(block, done) {
    this.block = block;
    this.done = done;
    this.end = __bind(this.end, this);
    this.join = __bind(this.join, this);
    this.part = __bind(this.part, this);
    if (typeof this.block === 'function') {
      this.done = this.block;
    }
    if (typeof this.block !== 'boolean') {
      this.block = false;
    }
    this.id = Join.ids++;
    this.count = 0;
    if (this.block) {
      this.part();
    }
  }

  Join.prototype.part = function(arg) {
    this.count++;
    if (typeof arg === 'function') {
      return setTimeout(arg, 0);
    }
  };

  Join.prototype.join = function() {
    if (--this.count === 0) {
      return this.done();
    }
  };

  Join.prototype.end = function(done) {
    this.done = done;
    return this.join();
  };

  return Join;

})();

if (typeof window !== "undefined" && window !== null) {
  window.Sync = Sync;
  window.Join = Join;
}

if ((typeof module !== "undefined" && module !== null) && (module.exports != null)) {
  module.exports.Sync = Sync;
  module.exports.Join = Join;
}
